@@ -6,13 +6,29 @@ import time
 import signal
 import builtins
 from subprocess import TimeoutExpired, check_output
-from io import BytesIO
 from collections import deque
 
-from xonsh.tools import ON_WINDOWS
+from xonsh.platform import ON_DARWIN, ON_WINDOWS, ON_CYGWIN
 
 tasks = deque()
 
+
+if ON_DARWIN:
+    def _send_signal(job, signal):
+        # On OS X, os.killpg() may cause PermissionError when there are
+        # any zombie processes in the process group.
+        # See github issue #1012 for details
+        for pid in job['pids']:
+            os.kill(pid, signal)
+
+elif ON_WINDOWS:
+    pass
+
+else:
+    def _send_signal(job, signal):
+        os.killpg(job['pgrp'], signal)
+
+
 if ON_WINDOWS:
     def _continue(job):
         job['status'] = "running"
@@ -60,9 +76,6 @@ else:
     def _kill(job):
         _send_signal(job, signal.SIGKILL)
 
-    def _send_signal(job, signal):
-        os.killpg(job['pgrp'], signal)
-
     def ignore_sigtstp():
         signal.signal(signal.SIGTSTP, signal.SIG_IGN)
 
@@ -81,10 +94,12 @@ else:
         #    give_terminal_to from bash 4.3 source, jobs.c, line 4030
         # this will give the terminal to the process group pgid
         if _shell_tty is not None and os.isatty(_shell_tty):
-            oldmask = signal.pthread_sigmask(signal.SIG_BLOCK,
-                                             _block_when_giving)
+            if not ON_CYGWIN:
+                oldmask = signal.pthread_sigmask(signal.SIG_BLOCK,
+                                                 _block_when_giving)
             os.tcsetpgrp(_shell_tty, pgid)
-            signal.pthread_sigmask(signal.SIG_SETMASK, oldmask)
+            if not ON_CYGWIN:
+                signal.pthread_sigmask(signal.SIG_SETMASK, oldmask)
 
     # check for shell tty
     try:
